/*StroboDuino sketch 170610 for 12x2 digit display on PWB 170423
 schematic 170530 
 Mike Huddleston
*/

#include <LiquidCrystal.h>                  // include LCD library
// format        (rs, en, d4, d5, d6, d7)    map of LCD assignments
LiquidCrystal lcd(12, 11, 10,  6,  5,  4);  // assign Arduino I/O to LCD

// declare digital I/O

#define pwDnPin 3             // pulse width 'down' button
#define by2Pin  7             // frequency (FPM) divide-by-2 button 
#define ledPin  8             // output to LED drive (flash)
#define dnPin   9             // frequency (FPM) 'down' button

// declare analog input

#define vBat    A1            // battery check (analog input)

// analog inputs repurposed as digital inputs from switches 

#define upPin   A0            // frequency (FPM) 'up' button             
#define x2Pin   A2            // frequency (FPM) multiply-by-2 button
#define pwUpPin A3            // pulse width 'up' button

// constant declarations for calculating timer loaders

long fXtal = 16.0E6;          // crystal frequency in Hz
long rXtal = 60.0*fXtal;      // crystal frequency in FPM

// variable declarations

long fpm = 3000;              // starting flash rate
float period;                 // intermediate result, timer loader for total period
unsigned long rLoaderH;       // register loader for OCR1A high phase
unsigned long rLoaderL;       // register loader for OCR1A low phase
float percentage = 0.0;       // default pulse width percentage
int fld = 200;                // foreground loop delay = 200ms

int upCtr = 0;                // switch debouncing counters
int dnCtr = 0;
int x2Ctr = 0;
int by2Ctr = 0;
int skipx2 = 0;
int skipby2 = 0;

int onDisplay = 0;            // suppress LCD update until first red key push
int splatOn;                  // displays splat if pulse width automatically extended

//*************************************************************************************

void setup()
{
  lcd.begin(12,2);               // LCD initial display
  lcd.clear();
  lcd.print ("StroboDuino*");
  lcd.setCursor (0,1);
  lcd.print ("Push red key");

  // initialize I/O pins

  pinMode(ledPin, OUTPUT);
  pinMode(upPin, INPUT_PULLUP);  // Assign button pins as inputs with pullups
  pinMode(pwUpPin, INPUT_PULLUP);
  pinMode(x2Pin, INPUT_PULLUP);
  pinMode(by2Pin, INPUT_PULLUP);
  pinMode(pwDnPin, INPUT_PULLUP);
  pinMode(dnPin, INPUT_PULLUP);

  digitalWrite (ledPin, LOW);    // initialize pulse output to 'off' state

  // initialize Timer 1 (16-bit timer with no sharing)

  noInterrupts();                // disable interrupts

  TCCR1A = 0;                    // normal timer 1 operation
  TCNT1 = 0;

  OCR1A = 100;                   // Dummy first interrupt; then calculated
  TCCR1B = B00001011;            // turn on WGM12, CS11, CS10 (divide 16MHz by 64)
  TIMSK1 = B00000010;            // enable timer compare interrupt

  interrupts();                  // re-enable interrupts
}
//************************************************************************************

//timer 1 interrupt service routine

ISR(TIMER1_COMPA_vect)
{
  // load appropriate delay for phase and invert output phase

  if (digitalRead(ledPin) == LOW) // if LED is off, load 'on' timer loader
  { 
    OCR1A = rLoaderH;
    digitalWrite (ledPin, HIGH);  // and turn LED on
  }
  else   
  {
    OCR1A = rLoaderL;             // load 'off' timer loader
    digitalWrite (ledPin, LOW);   // and turn LED off
  }
}
//************************************************************************************

void loop() 
{
  //  decode and execute button pushes

  if (digitalRead(pwUpPin) == LOW && digitalRead(pwDnPin) == LOW)
  { 
    percentage = 0.0;             // if both PW buttons are pushed, make PW=0
  }     
  else if (digitalRead(pwUpPin) == LOW)    // if pulse width up button is pushed
  {
    bumpPW();                     // add 0.5 to pulse width
    splatOn = 0;
    upCtr = 0;
    dnCtr = 0;
    onDisplay = 1;
  }
  else if (digitalRead(pwDnPin) == LOW)    // if pulse width down button
  {
    if (percentage > 0.0) percentage = percentage - 0.5;
    upCtr = 0;
    dnCtr = 0;
    splatOn = 0;
    onDisplay = 1;   
  }
  else if (digitalRead(upPin) == LOW)     // if FPM up button
  {
    upCtr += 1;
    if (upCtr > 60 && fpm < 380000) fpm += 10000; 
    else if (upCtr >40 && fpm < 398000) fpm += 1000;
    else if (upCtr >20 && fpm < 398800) fpm += 200;
    else if (upCtr >15 && fpm < 398900) fpm += 100;
    else if (upCtr >10 && fpm < 398990) fpm += 10;
    else if (upCtr > 5 && fpm < 398995) fpm += 5;
    else if (fpm < 399000) fpm = fpm + 1;
    dnCtr = 0;
  }
  else if (digitalRead(dnPin) == LOW)    // if FPM down button
  {
    dnCtr += 1;
    if (dnCtr > 60 && fpm > 10015) fpm -= 10000;
    else if (dnCtr > 40 && fpm > 1015) fpm -= 1000;
    else if (dnCtr > 20 && fpm > 215) fpm -= 200;
    else if (dnCtr > 15 && fpm > 115) fpm -= 100;
    else if (dnCtr > 10 && fpm >  25) fpm -= 10;
    else if (dnCtr >  5 && fpm >  20) fpm -= 5;
    else if (fpm > 15) fpm -= 1;
    upCtr = 0;
  }
  else if (digitalRead(x2Pin) == LOW && fpm < 199501)    // if FPM*2 button
  {
    if (skipx2 == 0)
    {
      fpm = 2*fpm;
      dnCtr = 0;
      upCtr = 0;  
      skipx2 = 1;
    }
  }
  else if (digitalRead(by2Pin) == LOW && fpm > 30)   // if FPM/2 button
  {
    if (skipby2 == 0)
    {
      fpm = fpm/2;
      dnCtr = 0;
      upCtr = 0;  
      skipby2 = 1;
    }
  }
  else                          // no button
  {
    dnCtr = 0;
    upCtr = 0;
    skipx2 = 0;
    skipby2 = 0;
  }
  CalcTimerConsts();            // calculate timer loaders based on new inputs
  delay(fld);                   // delay and loop back
}
//************************************************************************************

//Functions

void CalcTimerConsts ()
{
  if (fpm >= 14649.0)
  {
    TCCR1B = B00001001;          // turn on WGM12,CS10 (straight 16MHz, no prescale)
    period = rXtal / fpm;        //
  }
  else if (fpm >= 2000.0)
  {
    TCCR1B = B00001010;          // turn on WGM12,CS11 (divide 16MHz xtal by 8)
    period = (rXtal/8) / fpm;   //
  }
  else if (fpm >= 300.0)
  {
    TCCR1B = B00001011;          // turn on WGM12, CS11, CS10  (divide 16MHz xtal by 64)
    period = (rXtal/64) / fpm;   //
  }
  else if (fpm >= 100.0)
  {
    TCCR1B = B00001100;          // turn on WGM12, CS12 (divide 16MHz xtal by 256)
    period = (rXtal/256) / fpm;  //
  }
  else
  {
    TCCR1B = B00001101;          // turn on WGM12, CS10, CS12 (divide 16MHz xtal by 1024)
    period = (rXtal/1024) / fpm; //
  }

  calcLdrs();                    // calculate timer loaders based on new period calc

  while (percentage >0.0 && TCCR1B == B00001001 && rLoaderH < 236)    
  {
    bumpPW ();                   // if timer loader is too small, bump PW
    calcLdrs();
    splatOn = 1;                 // and display asterisk
  }

  if (onDisplay != 0)            // change display only if red button has been pushed  
  {
    lcd.clear();                 // update LCD
    lcd.setCursor (1,0);  
    lcd.print (String(fpm, DEC));// print FPM
    lcd.print (" FPM");

    lcd.setCursor (0,1);         // print pulse width percentage on second line
    lcd.print (percentage);
    lcd.setCursor (3,1);
    lcd.print ("%");
    if(splatOn == 1)
    {
      lcd.print("*");
    }
    else lcd.print (" "); 

    lcd.setCursor (7,1);
    lcd.print (float(analogRead(vBat)/75.0)); // print voltage
    lcd.setCursor (11,1);
    lcd.print( "v ");
  }
}

void  bumpPW ()                  // increase pulse width by 0.5%
{  
  if (percentage < 10.0) percentage = percentage + 0.5;
}

void calcLdrs ()                 // calc timer loaders from 'period' and 'percentage'  
{  
  rLoaderH = ((percentage / 100.0) * period); // calc timer loader for ON phase
  rLoaderL = .999*((1.0 - (percentage / 100.0)) * period); // calc tmr ldr for OFF phase
}

